@@ -18,6 +18,8 @@ from typing import (
 
 T = TypeVar("T")
 ConverterType = Callable[[str], Any]
+NoneType = type(None)
+UnionClass = Union[None, int].__class__
 
 
 def read_config(
@@ -268,6 +270,21 @@ def parse_bool(value: str) -> bool:
     return value.lower() in TEXT_TRUE_VALUES
 
 
+def unwrap_optional(typespec: Any) -> Optional[Any]:
+    if typespec.__class__ != UnionClass:
+        return None
+
+    union_args = [a for a in typespec.__args__ if a is not NoneType]
+
+    if len(union_args) != 1:
+        raise TypeError(
+            "Complex types mustn't be used in short form. You have to "
+            "specify argclass.Argument with converter or type function."
+        )
+
+    return union_args[0]
+
+
 def _make_action_true_argument(
     kind: typing.Type, default: Any = None,
 ) -> _Argument:
@@ -315,13 +332,25 @@ class Meta(ABCMeta):
             ):
                 attrs[key] = ...
 
+                is_required = argument is None
+
                 if _type_is_bool(kind):
                     argument = _make_action_true_argument(kind, argument)
                 else:
-                    argument = _Argument(type=kind, default=argument)
+                    optional_type = unwrap_optional(kind)
+                    if optional_type is not None:
+                        is_required = False
+                        kind = optional_type
+
+                    argument = _Argument(
+                        type=kind, default=argument, required=is_required,
+                    )
 
             if isinstance(argument, _Argument):
-                if argument.type is None:
+                if argument.type is None and argument.converter is None:
+                    if kind.__class__.__module__ == "typing":
+                        kind = unwrap_optional(kind)
+                        argument.default = None
                     argument.type = kind
                 arguments[key] = argument
             elif isinstance(argument, AbstractGroup):
@@ -407,9 +436,8 @@ class Parser(AbstractParser, Base):
         "See more https://pypi.org/project/argclass/#configs"
     )
 
-    @staticmethod
     def _add_argument(
-        parser: Any, argument: _Argument, dest: str, *aliases,
+        self, parser: Any, argument: _Argument, dest: str, *aliases,
     ) -> Tuple[str, Action]:
         kwargs = argument.get_kwargs()
 
@@ -429,6 +457,9 @@ class Parser(AbstractParser, Base):
                 f"{kwargs.get('help', '')} [ENV: {argument.env_var}]"
             ).strip()
 
+            if argument.env_var in os.environ:
+                self._used_env_vars.add(argument.env_var)
+
         return dest, parser.add_argument(*aliases, **kwargs)
 
     @staticmethod
@@ -465,6 +496,7 @@ class Parser(AbstractParser, Base):
         self._epilog += self.HELP_APPENDIX_END
         self._auto_env_var_prefix = auto_env_var_prefix
         self._parser_kwargs = kwargs
+        self._used_env_vars: Set[str] = set()
 
     def _make_parser(
         self, parser: Optional[ArgumentParser] = None,
@@ -586,6 +618,7 @@ class Parser(AbstractParser, Base):
                     )
 
     def parse_args(self, args: Optional[Sequence[str]] = None) -> "Parser":
+        self._used_env_vars.clear()
         parser, destinations = self._make_parser()
         parsed_ns = parser.parse_args(args)
 
@@ -612,10 +645,15 @@ class Parser(AbstractParser, Base):
 
         return self
 
-    def print_help(self):
+    def print_help(self) -> None:
         parser, _ = self._make_parser()
         return parser.print_help()
 
+    def sanitize_env(self) -> None:
+        for name in self._used_env_vars:
+            os.environ.pop(name, None)
+        self._used_env_vars.clear()
+
 
 # noinspection PyPep8Naming
 def Argument(
